""" JSR-170 ItemDefinition interface, Level 1

$Id: item_definition.py 515 2005-07-11 18:00:47Z tseaver $
"""
from zope.interface import Interface

class IItemDefinition(Interface):
    """ Abstract base for INodeDefinition and IPropertyDefinition.
    """

    def getDeclaringNodeType():
        """ Return the "parent" INodeType that contains the declaration of
            this instance.
        """

    def getName():
        """ Return the name of the item to which this definition applies.

        o If the name is "*", then this definition defines a *residual set*
          of child items;  that is, it defines the characteristics of all
          those child items with names *apart from the names explicitly
          used in other item definitions*.
        """

    def isAutoCreated():
        """ Return True if the item is to be automatically created when
            its parent node is created, else False

        o If True then this ItemDefinition will necessarily not be a *residual
          set definition* but will specify an actual item name
          (in other words getName() will not return "*").
        """

    def isMandatory():
        """ Return True if the item is mandatory, else False.

        o A mandatory item is one that, if its parent node exists, must also
          exist. This means that a mandatory single-value property must have
          a value (since there is no such thing a null value).

        o In the case of multi-value properties this means that the property
          must exist, though it can have zero or more values.

        o An attempt (in a level 2 implementation) to save a node that has
          a mandatory child item without first creating that child item will
          throw a ConstraintViolationException on save.
        """

    def getOnParentVersion():
        """ Return the IOnParentVersion status of the property.

        o This flag governs what occurs (in implementations that support
          versioning) when the parent node of this item is checked in.
        """

    def isProtected():
        """ Return True if the child item is protected, else False.

        o In level 2 implementations, a protected item is one that cannot be
          removed (except by removing its parent) or modified directly through
          this API (that is, 'IItem.remove', 'INode.addNode',
          'INode.setProperty' and 'IProperty.setValue').

        o A protected node may be removed or modified (in a level 2
          implementation), however, through some mechanism not defined by this
          specification or as a side-effect of operations other than the
          standard write methods of the API.
        """
