""" JSR-170 NamespaceRegistry interface, Level 1

$Id: namespace_registry.py 515 2005-07-11 18:00:47Z tseaver $
"""
from zope.interface import Interface


class INamespaceRegistry(Interface):
    """ Map prefixes <--> namespace URIs.

    o "Stock" (required) mappings include:

      - 'jcr' <--> 'http://www.jcp.org/jcr/1.0'

        Reserved for items defined within built-in node types.
        For example 'jcr:content'.

      - 'nt' <--> 'http://www.jcp.org/jcr/nt/1.0'

        Reserved for the names of built-in primary node types.

      - 'mix' <--> 'http://www.jcp.org/jcr/mix/1.0'

        Reserved for the names of built-in mixin node types.

      - 'xml' <--> 'http://www.w3.org/XML/1998/namespace'

        Reserved for reasons of compatibility with XML. This prefix should
        not be used by clients of the API in the names of normal nodes or
        properties, since doing so will cause problems on export to XML.

      - '' (the empty prefix) -> '' (the empty URI)

        This makes the default namespace the empty URI.  In effect this
        means that a name without a prefix is identical in both its prefixed
        form and in its fully qualified form (i.e. when it is stored
        internally as URI plus local name).

    """
    def getPrefixes():
        """ Return a sequence of all currently registered prefixes.
        
        o Raise RepositoryException if an error occurs.
        """

    def getURIs():
        """ Return a sequence of all currently registered URIs.
        
        o Raise RepositoryException if an error occurs.
        """

    def getURI(prefix):
        """ Return the URI to which the given prefix is mapped.
        
        o Raise NamespaceException if a mapping with the specified prefix
          does not exist.
          
        o Raise RepositoryException if another error occurs.
        """

    def getPrefix(uri):
        """ Return the prefix which is mapped to the given uri.
        
        o Raise NamespaceException if a mapping with the specified uri does
          not exist.
          
        o Raise RepositoryException if another error occurs.
        """
