""" JSR-170 NodeDefinition interface, Level 1

$Id: node_definition.py 515 2005-07-11 18:00:47Z tseaver $
"""
from zope.interface import Interface

from jsr170.level1 import IItemDefinition

class INodeDefinition(IItemDefinition):
    """ A node definition specifies the required node types, the primary type,
        and same-name constraints of a given child node.
    """

    def getRequiredPrimaryTypes():
        """ Return a sequence of the minimum set of required primary node
            types that the child node must have.

        o Returns an array to support those implementations with multiple
          inheritance.

        o This method never returns an empty array. If this node definition
          places no requirements on the primary node type, then this method
          will return an array containing only the NodeType object
          representing 'nt:base', which is the base of all primary node types
          and therefore constitutes the least restrictive node type requirement.

        o Note that any particular node instance still has only one assigned
          primary node type, but in multiple-inheritance-supporting
          implementations the RequiredPrimaryTypes attribute can be used to
          restrict that assigned node type to be a subtype of all of a
          specified set of node types.
        """

    def getDefaultPrimaryType():
        """ Return the default node type that will be assigned to the
            child node if it is created without an explicitly specified
            node type.

        o This node type must be a subclass of (or the same class as) the node
          type(s) returned by 'getRequiredPrimaryTypes'.

        o A return value of None indicates that no default primary type is
          specified and that therefore an attempt to create this node without
          specifying a node type will throw a ConstraintViolationException.

        o Note that to indicate a None value for this attribute in a node
          type definition that is stored in content, the
          'jcr:defaultPrimaryType' property is simply removed (since null
          values for properties are not allowed.
        """

    def allowsSameNameSiblings():
        """ Return True if this child node can have same-name siblings, else
            False.

        o In other words, report whether the parent node can have more than
          one child node of this name.
        """
