""" JSR-170 NodeType interface, Level 1

$Id: node_type.py 514 2005-07-11 17:57:16Z tseaver $
"""
from zope.interface import Interface

class INodeType(Interface):
    """ A node type specifies allowable properties and child nodes of a node.
    """

    def getName():
        """ Return the name of the node type.
        """

    def isMixin():
        """ Return True if this is a mixin type; else False (primary).
        """

    def hasOrderableChildNodes():
        """ Return True if nodes of this type must support orderable child
           nodes; else False.
           
        o If a node type returns True on a call to this method, then all
          nodes of that node type must support the method 'Node.orderBefore'.
          
        o If a node type returns False on a call to this method, then nodes of
          that node type *may* support 'Node.orderBefore'.
          
        o Only the primary node type of a node controls that node's status in
          this regard.  This setting on a mixin node type will not have any
          effect on the node.
        """

    def getPrimaryItemName():
        """ Return the name of the primary item (one of the child items of
            the nodes of this node type).
        
        o If this node has no primary item, then this method returns None.
        
        o This indicator is used by the method 'Node.getPrimaryItem()'.
        """

    def getSupertypes():
        """ Return a sequence of all supertypes of this node type in the
            node type inheritance hierarchy.
            
        o For primary types apart from 'nt:base', this list will always
          include at least 'nt:base'.
          
        o For mixin types, there is no required supertype.
        """

    def getDeclaredSupertypes():
        """ Return the direct supertypes of this node type in the node type
            inheritance hierarchy, that is, those actually declared in this
            node type.
        
        o In singleinheritance systems, this will always be an array of size
          0 or 1.
          
        o In systems that support multiple inheritance of node types this
          array may be of size greater than 1.
        """

    def isNodeType(nodeTypeName):
        """ Return True if this node type is 'nodeTypeName' or a subtype of
            'nodeTypeName';  else False.
        """
    
    def getPropertyDefinitions():
        """ Return a sequence containing the property definitions of this
            node type.

        o The returned sequence includes both those property definitions
          actually declared in this node type and those inherited from the
          supertypes of this type.
        """
    
    def getDeclaredPropertyDefinitions():
        """ Return a sequence containing the property definitions actually
            declared in this node type.
        """

    def getChildNodeDefinitions():
        """ Return a sequence containing the child node definitions of
            this node type.
        
        o The returned sequence includes both those child node definitions
          actually declared in this node type and those inherited from the
          supertypes of this node type.
        """

    def getDeclaredChildNodeDefinitions():
        """ Return a sequence containing the child node definitions
            actually declared in this node type.
        """

    def canSetProperty(propertyName, value=None, values=None):
        """ Return True if setting 'propertyName' to 'value' is allowed by
            this node type;  else False.

        o One of either 'value' or 'values', but not both, must be passed.
        
        o 'value', if passed, must be an IValue instance.
        
        o 'values', if passed, must be a sequence of IValue instances.
        """

    def canAddChildNode(childNodeName, nodeTypeName=None):
        """ Return True if this node type allows the addition of a
            child node called 'childNodeName';  else False.

        o 'childNodeName' is the name under which the node is to be added.

        o 'nodeTypeName' is the name of the node type.
            
        o If 'nodeTypeName' is None, then in the definition of this parent
          node type, the child node name must be sufficient to determine
          the intended child node type, else return False.
        """

    def canRemoveItem(itemName):
        """ Return True if removing the child item called 'itemName' is
            allowed by this node type;  else False.
        """
