""" JSR-170 Property interface. Level 1

$Id: property.py 515 2005-07-11 18:00:47Z tseaver $
"""
from jsr170.level1.item import IItem

class IProperty(IItem):
    """ A property represents a simple, named "leaf" value of a node.
    """
    def getValue():
        """ Return the value of this property as a Value object.

        o Raise ValueFormatException if this property is multi-valued.

        o The object returned is a copy of the stored value and is immutable.

        o Raise RepositoryException if an error occurs.
        """

    def getValues():
        """ Return a sequence of all the values of this property.

        o This method is used to access multi-value properties.

        o The sequence returned is a copy of the stored values, so changes to
          it are not reflected in internal storage.

        o Raise ValueFormatException if the property is single-valued.

        o Raise RepositoryException if an error occurs.
        """

    def getString():
        """ Return a string representation of the value of this property.

        o A shortcut for Property.getValue().getString().

        o Raise ValueFormatException if this property is multi-valued, or
          if the value of this property cannot be converted to a string.

        o Raise RepositoryException if another error occurs.
        """

    def getStream():
        """ Return an InputStream representation of the value of this property.

        o This method is a shortcut for 'Property.getValue().getStream()'.

        o Raise ValueFormatException if this property is multi-valued.

        o Raise RepositoryException if another error occurs.
        """

    def getLong():
        """ Return an int representation of the value of this property.

        o A shortcut for Property.getValue().getLong().

        o Raise ValueFormatException if this property is multi-valued, or
          if the value of this property cannot be converted to an int.

        o Raise RepositoryException if another error occurs.
        """

    def getDouble():
        """ Return a float representation of the value of this property.

        o A shortcut for Property.getValue().getDouble().

        o Raise ValueFormatException if this property is multi-valued, or
          if the value of this property cannot be converted to a float.

        o Raise RepositoryException if another error occurs.
        """

    def getDate():
        """ Return a datetime representation of the value of this property.

        o A shortcut for Property.getValue().getDate().

        o Raise ValueFormatException if this property is multi-valued, or
          if the value of this property cannot be converted to a datetime.

        o Raise RepositoryException if another error occurs.
        """

    def getBoolean():
        """ Return a bool representation of the value of this property.

        o A shortcut for Property.getValue().getBoolean().

        o Raise ValueFormatException if this property is multi-valued, or
          if the value of this property cannot be converted to a bool.

        o Raise RepositoryException if another error occurs.
        """

    def getNode():
        """ Return the node to which this property refers.

        o Only valid for REFERENCE-type properties.

        o Raise ValueFormatException if this property is multi-valued, or
          if the value of this property cannot be converted to a reference,
          or if the property is currently part of the frozen state of a
          version in version storage.

        o Raise RepositoryException if another error occurs.
        """

    def getLength():
        """ Return the length of the value of this property in bytes if
            the value is a PropertyType.BINARY;  otherwise it returns the
            number of characters needed to display the value in its string
            form.

        o Returns 1 if the implementation cannot determine the length of
          the value.

        o Raises ValueFormatException if this property is multi-valued.

        o Raises RepositoryException if another error occurs.
        """

    def getLengths():
        """ Return a sequence holding the lengths of the values of this
           (multi-value) property in bytes if the values are
           PropertyType.BINARY;  otherwise it returns the number of characters
           needed to display each value in its string form.

        o The order of the length values corresponds to the order of the
          values in the property.

        o Returns a  1 in the appropriate position if the implementation
          cannot determine the length of a value.

        o Raises ValueFormatException if this property is single-valued.

        o Raises RepositoryException if another error occurs.
        """

    def getType():
        """ Return the type of this Property.

        o The type returned is that which was set at property creation.

        o Note that for some property 'p', the type returned by 'p.getType()'
          may differ from the type returned by
          'p.getDefinition.getRequiredType()' only in the case where the
          latter returns UNDEFINED. The type of a property instance is never
          UNDEFINED (it must always have some actual type).
        """

    def getDefinition():
        """ Return the property definition that applies to this Property.

        o In some cases there may appear to be more than one definition
          that could apply to this property.  However, it is assumed that
          upon creation of this property, a single particular definition was
          used and it is that definition that this method returns.

        o How this governing definition is selected upon property creation
          from among others which may have been applicable is an
          implementation issue and is not covered by this specification.

        o Raise RepositoryException if an error occurs.
        """
