""" JSR-170 PropertyDefinition interface. Level 1

$Id: property_definition.py 515 2005-07-11 18:00:47Z tseaver $
"""
from zope.interface import Interface

from jsr170.level1 import IItemDefinition

class IPropertyDefinition(IItemDefinition):
    """ A property definition specifieds the name, allowed type, value
        constraints, and protection of a given property.
    """

    def getRequiredType():
        """ Return the required type of the property.

        o One of the values enumerated in IPropertyType (STRING, BINARY,
         DATE, LONG, DOUBLE, NAME, PATH, REFERENCE, BOOLEAN or UNDEFINED).

        o If UNDEFINED, then this property may be of any type.
        """

    def getValueConstraints():
        """ Return a sequence of constraint strings.

        o The returned sequence describes the constraints that exist on
          values of the property.

        o Reporting of value constraints is optional. An implementation may
          return None, indicating that value constraint information is
          unavailable (though a constraint may still exist).

        o Note that to indicate a None value for this attribute in a
          node type definition that is stored in content, the
          'jcr:valueConstraints' property is simply removed (since null values
          for properties are not allowed).

        o Returning an empty array, on the other hand, indicates that
          constraint information is available and that no constraints are
          placed on the value of the property.

        o If a non-empty array is returned then it is interpreted as a
          disjunctive set of constraints (i.e. the value must meet at least
          one of the constraints). The interpretation of the constraint
          strings themselves differs according to the type of the property.
        """

    def getDefaultValues():
        """ Return the default value(s) of the property.

        o The object(s) returned are the values (or value) that the property
          defined by this PropertyDefinition will be assigned if it is
          automatically created (that is, if 'isAutoCreated()' returns True).

        o This method returns a sequence of IValue objects.

        o If the property is multi-valued, then the sequence represents the
          full set of values that the property will be assigned upon being
          auto-created. Note that this could be the empty array.

        o If the property is single-valued, then the array returned will be of
          size 1.

        o If None is returned, then the property has no fixed default value.
          This does not exclude the possibility that the property still
          assumes some value automatically, but that value may be
          parameterized (for example,  the current date ) and hence not
          expressible as a single fixed value.  In particular, this must be
          the case if 'isAutoCreated' returns True and this method returns None.

        o Note that to indicate a None value for this attribute in a node type
          definition that is stored in content, the 'jcr:defaultValues'
          property is simply removed (since null values for properties are
          not allowed.
        """

    def isMultiple():
        """ Return True if this property can have multiple values, else False.

        o Note that the 'isMultiple' flag is special in that a given node
          type may have two property definitions that are identical in every
          respect except for the their 'isMultiple' status. For example,
          a node type can specify two string properties both called 'X',
          one of which is multi-valued and the other not. An example of
          such a node type is 'nt:unstructured'.
        """
