""" JSR-170 PropertyType interface. Level 1

$Id: property_type.py 515 2005-07-11 18:00:47Z tseaver $
"""
from zope.interface import Interface
from zope.interface import Attribute

class IPropertyType(Interface):
    """ Enumerate the allowable values for a property's 'type'.
    """

    STRING = Attribute('STRING',
        """jcr.property.types.string

        o The STRING property type is used to store strings.
        
        o The property value will be an instance of Python's 'unicode'
          type (or a subclass).

        o type is int
        
        o value is 0
        """
        )

    BINARY = Attribute('BINARY',
        """jcr.property.types.binary

        o BINARY properties are used to store binary data. 
        
        o The property value will be an instance of Pythons 'str' type
          (or a subclass).

        o type is int

        o value is 1
        """
        )

    LONG = Attribute('LONG',
        """jcr.property.types.long

        o The LONG property type is used to store integers.
        
        o The property value will be an instance of Python's 'long' type
          (or a subclass).
        
        o type is int
        
        o value is 2
        """
        )

    DOUBLE = Attribute('DOUBLE',
        """jcr.property.types.double

        o The DOUBLE property type is used to store floating point numbers.

        o The property value will be an instance of Python's 'float' type
          (or a subclass).

        o type is int

        o value is 3
        """
        )

    BOOLEAN = Attribute('BOOLEAN',
        """jcr.property.types.boolean

        o The BOOLEAN property type is used to store boolean values.

        o The property value will be an instance of Python's 'bool' type
          (or a subclass).

        o type is int

        o value is 4
        """
        )

    DATE = Attribute('DATE',
        """jcr.property.types.date

        o The DATE property type is used to store time and date information.

        o The property value will be an instance of Python's 'datetime.date'
          type (or a subclass, e.g., 'datetime.datetime').

        o type is int

        o value is 5
        """
        )

    NAME = Attribute('NAME',
        """jcr.property.types.name

        o A NAME is a pairing of a namespace and a local name.
        
        o When read, the namespace is mapped to the current prefix.
        
        o The property value will be an instance of Python's 'unicode'
          type (or a subclass), in the form, '%(prefix)s:%(local_name)s'.

        o type is int

        o value is 6
        """
        )

    PATH = Attribute('PATH',
        """jcr.property.types.path

        o A PATH property is an ordered list of path elements.
        
        o A path element is a NAME with an optional index.
        
        o When read, the NAMEs within the path are mapped to their current
          prefix.
          
        o A path may be absolute (beginning with an empty NAME) or relative.

        o The property value will be an tuple, whose elements are of the
          same type / format as NAME properties.

        o type is int

        o value is 7
        """
        )

    REFERENCE = Attribute('REFERENCE',
        """jcr.property.types.reference

        o A REFERENCE property stores the UUID of a referenceable node
          (one having type "mix:referenceable"), which must exist within the
          same workspace or session as the REFERENCE property.

        o A REFERENCE property enforces this referential integrity by
          preventing (in level 2 implementations) the removal of its target
          node.

        o The property value will be an instance of Python's 'basestring'
          type, or a subclass.

        o type is int

        o value is 8
        """
        )

    UNDEFINED  = Attribute('UNDEFINED',
        """jcr.property.types.undefined

        o This constant can be used within a property definition to specify
          that the property in question may be of any type. However, it cannot
          be the actual type of any property instance.
          
          For example it will never be returned by 'Property.getType' and
          (in level 2 implementations) it cannot be assigned as the type
          when creating a new property.

        o type is int

        o value is 9
        """
        )

    # These constants define the standard string forms of the property types.
    # These are used, for example, when serializing content to XML.
    TYPENAME_STRING = Attribute('TYPENAME_STRING', "String")
    TYPENAME_BINARY = Attribute('TYPENAME_BINARY', "Binary")
    TYPENAME_LONG = Attribute('TYPENAME_LONG', "Long")
    TYPENAME_DOUBLE = Attribute('TYPENAME_DOUBLE', "Double")
    TYPENAME_DATE = Attribute('TYPENAME_DATE', "Date")
    TYPENAME_BOOLEAN = Attribute('TYPENAME_BOOLEAN', "Boolean")
    TYPENAME_NAME = Attribute('TYPENAME_NAME', "Name")
    TYPENAME_PATH = Attribute('TYPENAME_PATH', "Path")
    TYPENAME_REFERENCE = Attribute('TYPENAME_REFERENCE', "Reference")
    TYPENAME_UNDEFINED = Attribute('TYPENAME_UNDEFINED', "Undefined")

    def nameFromValue(type):
        """ Return the standard name of the given property type,
            specified by its integer value.
        """

    def valueFromName(name):
        """ Return the integer value of the given property type,
            specified by its standard name.
        """
