""" JSR-170 Query interface. Level 1

$Id: query.py 515 2005-07-11 18:00:47Z tseaver $
"""
from zope.interface import Interface
from zope.interface import Attribute

class IQuery(Interface):
    """
    """
    def execute():
        """ Execute this query and returns an IQueryResult.

        o Raise RepositoryException if an error occurs.
        """

    def getStatement():
        """ Return the statement defined for this query.
        """

    def getLanguage():
        """ Return the language set for this query.

        o This will be one of the strings returned by
          QueryManager.getSupportedQueryLanguages().
        """

    def getStoredQueryPath():
        """ Return the path of the node which contains this query.

        o If this is a Query that has been stored using 'Query.storeAsNode'
          (regardless of whether it has been saved yet) or retrieved using
          QueryManager.getQuery, then this method returns the path of the
          'nt:query' node that stores the query.

        o Raise ItemNotFoundException if this is a transient query (that is,
          a Query object created with 'QueryManager.createQuery' and not yet
          stored).

        o Raise RepositoryException if another error occurs.
        """

    XPATH = Attribute('XPATH',
        """jcr.query.xpath',

        o A string constant representing the XPath query language applied
          to the document view XML mapping of the workspace.

        o type is unicode

        o value is u'XPATH'
        """
        )

    SQL = Attribute('SQL',
        """jcr.query.sql',

        o A string constant representing the SQL query language applied to
          the database view of the workspace.

        o Support for this language is optional.

        o type is unicode

        o value is u'SQL'
        """
        )

