""" JSR-170 QueryManager interface. Level 1

$Id: query_manager.py 515 2005-07-11 18:00:47Z tseaver $
"""
from zope.interface import Interface

class IQueryManager(Interface):
    """ The QueryManager object provides methods for creating queries,
        retrieving saved queries and for discovering supported query
        languages.
    """
    def createQuery(statement, language):
        """ Create a new query from the specified query statement and language.
            
        o 'statement' is a string, containing the actual query.

        o 'language' is a string, identifying the syntax used to express the
          query.

        o Raise InvalidQueryException if the query 'statement' is syntactically
          invalid, given the 'language' specified.
          
        o Raise InvalidQueryException if the language parameter is not one of
          the strings returned by 'getSupportedQueryLanguages()'.
          
        o Raise RepositoryException if another error occurs.
        """
    def getQuery(node):
        """ Retrieve an existing persistent query.
        
        o Raise InvalidQueryException if node is not a valid persisted query
          (that is, a node of type 'nt:query').
          
        o Persistent queries are created by first using 'createQuery' to
          create a Query object and then calling 'save' to persist the query
          to a location in the workspace.
          
        o Raise RepositoryException if another error occurs.
        """
        
    def getSupportedQueryLanguages():
        """ Return a sequence of strings identifying supported query languages.

        o In level 1 this set must include the string represented by the
          constant Query.XPATH.
          
        o If SQL is supported it must additionally include Query.SQL.
        
        o An implementation of either level may also support other
          languages.
          
        o Raise RepositoryException if an error occurs.
        """

