""" JSR-170 Value interface. Level 1

$Id: value.py 515 2005-07-11 18:00:47Z tseaver $
"""
from zope.interface import Interface
from zope.interface import Attribute

class IValue(Interface):
    """ A Value represents an immutable value for a property.
    """
    def getString():
        """ Return a string representation of the value.

        o Raise ValueFormatException if this value cannot be converted
          to a string.

        o Raise IllegalStateException if 'getStream' has previously been
          called on this instance.  In this case, a new Value instance must
          be acquired in order to successfully call 'getString'.

        o Raise RepositoryException if another error occurs.
        """

    def getStream():
        """ Return an InputStream representation of the value of this property.

        o Raise IllegalStateException if a non-stream 'get' method has
          previously been called on this instance.  In this case, a new Value
          instance must be acquired in order to successfully call 'getStream'.

        o Raise RepositoryException if another error occurs.
        """

    def getLong():
        """ Return an int representation of the value of this property.

        o Raise ValueFormatException if this value cannot be converted to
          an int.

        o Raise IllegalStateException if 'getStream' has previously been
          called on this instance.  In this case, a new Value instance must
          be acquired in order to successfully call 'getLong'.

        o Raise RepositoryException if another error occurs.
        """

    def getDouble():
        """ Return a float representation of the value of this property.

        o A shortcut for Property.getValue().getDouble().

        o Raise ValueFormatException if this  value cannot be converted to
          a float.

        o Raise IllegalStateException if 'getStream' has previously been
          called on this instance.  In this case, a new Value instance must
          be acquired in order to successfully call 'getDouble'.

        o Raise RepositoryException if another error occurs.
        """

    def getDate():
        """ Return a datetime representation of the value of this property.

        o A shortcut for Property.getValue().getDate().

        o Raise ValueFormatException if this value cannot be converted to
          a datetime.

        o Raise IllegalStateException if 'getStream' has previously been
          called on this instance.  In this case, a new Value instance must
          be acquired in order to successfully call 'getDate'.

        o Raise RepositoryException if another error occurs.
        """

    def getBoolean():
        """ Return a bool representation of the value of this property.

        o A shortcut for Property.getValue().getBoolean().

        o Raise ValueFormatException if this property is multi-valued, or
          if the value of this property cannot be converted to a bool.

        o Raise IllegalStateException if 'getStream' has previously been
          called on this instance.  In this case, a new Value instance must
          be acquired in order to successfully call 'getString'.

        o Raise RepositoryException if another error occurs.
        """

    def getType():
        """ Return the type of this value. 
        
        o The type returned is that which was set at property creation.
        """

    def getLength():
        """ Return the length of the value of this property in bytes if
            the value is a PropertyType.BINARY;  otherwise it returns the
            number of characters needed to display the value in its string
            form.

        o Returns 1 if the implementation cannot determine the length of
          the value.

        o Raises ValueFormatException if this property is multi-valued.

        o Raises RepositoryException if another error occurs.
        """

    def getLengths():
        """ Return a sequence holding the lengths of the values of this
           (multi-value) property in bytes if the values are
           PropertyType.BINARY;  otherwise it returns the number of characters
           needed to display each value in its string form.

        o The order of the length values corresponds to the order of the
          values in the property.

        o Returns a  1 in the appropriate position if the implementation
          cannot determine the length of a value.

        o Raises ValueFormatException if this property is single-valued.

        o Raises RepositoryException if another error occurs.
        """

    def getType():
        """ Return the type of this Property.

        o The type returned is that which was set at property creation.

        o Note that for some property 'p', the type returned by 'p.getType()'
          may differ from the type returned by
          'p.getDefinition.getRequiredType()' only in the case where the
          latter returns UNDEFINED. The type of a property instance is never
          UNDEFINED (it must always have some actual type).
        """
