""" JSR-170 Workspace interface. Level 1

$Id: workspace.py 515 2005-07-11 18:00:47Z tseaver $
"""
from zope.interface import Interface


class IWorkspace(Interface):
    """ Represent a "view" of an actual repository workspace
        entity as seen through the authorization settings of its associated
        ISession.

    o Each IWorkspace object is associated one-to-one with an ISession
      object. The IWorkspace object can be acquired by calling
      'getWorkspace().' on the associated ISession object.
    """
    def getSession():
        """ Return the ISession object through which this workspace was
            acquired.
        """

    def getName():
        """ Return the name of the persistent workspace represented by this
            workspace.
        """

    def getQueryManager():
        """ Return an IQueryManager through which this workspace may be
            searched.

        o Raise RepositoryException if an error occurs.
        """

    def getNamespaceRegistry():
        """ Return an INamespaceRegistry used to access information and
            (in level 2) set the mapping between namespace prefixes and URIs.

        o Raise RepositoryException if another error occurs.
        """

    def getNodeTypeManager():
        """ Return an INodeTypeManager through which node type information
            can be queried.

        o There is one node type registry per repository.  Therefore the
          returned INodeTypeManager is not workspace-specific;  it provides
          introspection methdos for the global, repository-wide set of
          available node types.

        o Raise RepositoryException if another error occurs.
        """

    def getAccessibleWorkspaceNames():
        """ Return an sequence of names of all workspaces in this repository
            that are accessible to this user, given the ICredentials that
            were used to get the ISession tied to this workspace.

        o In order to access one of the listed workspaces, the user performs
          another 'IRepository.login', specifying the name of the desired
          workspace, and receives a new ISession object.
        """
