""" JSR-170 NamespaceRegistry interface, Level 2

$Id: namespace_registry.py 516 2005-07-11 18:08:28Z tseaver $
"""
from jsr170.level1 import INamespaceRegistry as INamespaceRegistryLevel1

class INamespaceRegistry(INamespaceRegistryLevel1):
    """ Extend INamespaceRegistryLevel1 to allow writes.
    """
    def registerNamespace(prefix, uri):
        """ Set a one-to-one mapping between prefix and uri in the
            global namespace registry of this repository.
            
        o Assigning a new prefix to a URI that already exists in the
         namespace registry erases the old prefix. In general this can be
         done, though an implementation is free to prevent particular
         remappings by throwing a NamespaceException.
         
        o On the other hand, taking a prefix that is already assigned to
          a URI and re-assigning it to a new URI in effect unregisters
          that URI. Therefore, the same restrictions apply to this operation
          as to 'NamespaceRegistry.unregisterNamespace':
         
          - Attempting to re-assign a built-in prefix (jcr, nt, mix, xml or
            the empty prefix) to a new URI will throw a NamespaceException."
         
          - Attempting to re-assign a prefix that is currently assigned to a
            URI that is present in content (either within an item name or
            within the value of a NAME or PATH property) will throw a
            NamespaceException.  This includes prefixes in use within
            in-content node type definitions.

          - Attempting to register a namespace with a prefix that begins with
            the characters 'xml'  (in any combination of case) will throw a
            NamespaceException.
          
          - An implementation may prevent the re-assignment of any other
            namespace prefixes for implementation-specific reasons by throwing
            a NamespaceException.
          
        o In a level 1 implementation, this method always throws an
          UnsupportedRepositoryOperationException.
          
        o Raise AccessDeniedException if the session associated with the
          Workspace object through which this registry was acquired does
          not have sufficient permissions to register the namespace.
          
        o Raise RepositoryException if another error occurs.
        """

    def unregisterNamespace(prefix):
        """ Remove a namespace mapping from the registry.
        
        o The following restrictions apply:

          - Attempting to unregister a built-in namespace (jcr, nt, mix,
            xml or the empty namespace) will throw a NamespaceException.
            
          - Attempting to unregister a namespace that is currently present
            in content (either within an item name or within the value of a
            NAME or PATH property) will throw a NamespaceException.
            This includes prefixes in use within in-content node type
            definitions. 

          - An attempt to unregister a namespace that is not currently
            registered will throw a NamespaceException. 
            
          - An implementation may prevent the unregistering of any other
            namespace for implementation-specific reasons by throwing a
            NamespaceException.
            
        o In a level 1 implementation, this method always throws an
          UnsupportedRepositoryOperationException.
          
        o Raise AccessDeniedException if the session associated with the
          Workspace object through which this registry was acquired does
          not have sufficient permissions to unregister the namespace.
          
        o Raise RepositoryException if another error occurs.
        """

