""" JSR-170 Workspace interface, Level 2

$Id: workspace.py 514 2005-07-11 17:57:16Z tseaver $
"""
from jsr170.level1 import IWorkspace as IWorkspaceLevel1

class IWorkspace(IWorkspaceLevel1):
    """ Extend IWorkspaceLevel1 to allow writes.
    """
    def copy(srcAbsPath, destAbsPath, srcWorkspace=None):
        """ Copy the node at 'srcAbsPath' to the new location at
            'destAbsPath'.

        o 'srcAbsPath' is the path of the node to be copied.

        o 'destAbsPath' is the location to which the node at 'srcAbsPath'
          is to be copied.

        o 'srcWorspace', if passed, is the name of the workspace in which
          to find the source node isuing 'srcAbsPath'.  If 'srcWorspace' is
          None, then both source and destination paths refer to the current
          workspace.

        o If successful, the change is persisted immediately;  there is no
          need to call 'save'.

        o Copies of referenceable nodes (nodes with UUIDs) are automatically
          given new UUIDs.

        o Raise NoSuchWorkspaceException if 'srcWorspace' is passed, but
          does not correspond to a known workspace within this repository.

        o Raise RepositoryException if the 'destAbsPath' provided has 
          an index on its final element.  Strictly speaking, the 'destAbsPath'
          parameter is actually an *absolute path* to the parent node of the
          new location, appended with the new *name* desired for the moved
          node. It does not specify a position within the child node ordering
          (if such ordering is supported).  If ordering is supported by the
          node type of the parent node of the new location, then the newly
          moved node is appended to the end of the child node list.

        o This method cannot be used to copy just an individual property by
          itself.  It copies an entire node and its subtree (including, of
          course, any properties contained therein).

        o Raise ConstraintViolationException if the operation would violate
          a node type or other implementation-specific constraint.

        o Raise VersionException if the parent node of 'destAbsPath' is
          versionable and checked-in, or is non-versionable
          but its nearest versionable ancestor is checked-in.

        o Raise AccessDeniedException if the current session (i.e. the session
          that was used to acquire this 'Workspace' object) does not have
          sufficient access rights to complete the operation.

        o Raise PathNotFoundException if the note at 'srcAbsPath' or the
          parent of 'destAbsPath' does not exist.

        o Raise ItemExistsException if a property already exists at
          'destAbsPath' or a node already exists there, and same-name siblings
          are not allowed.

        o Raise LockException if a lock prevents the copy.
        """

    def clone(srcWorspace, srcAbsPath, destAbsPath, removeExisting):
        """ Clone the node at 'srcAbsPath' to the new location at
            'destAbsPath'.

        o 'srcWorspace', is the name of the workspace in which
          to find the source node isuing 'srcAbsPath'. 

        o 'srcAbsPath' is the path of the node to be copied.

        o 'destAbsPath' is the location to which the node at 'srcAbsPath'
          is to be copied.

        o 'removeExisting' is a boolean flag:
        
          - If 'removeExisting' is True and an existing node in this workspace
            (the destination workspace) has the same UUID as a node being
            cloned from 'srcWorkspace', then the incoming node takes
            precedence, and the existing node (and its subtree) is removed.
          
          - If 'removeExisting' is False then a UUID collision causes this
            method to raise ItemExistsException and no changes are made.

        o If successful, the change is persisted immediately;  there is no
          need to call 'save'.

        o Copies of referenceable nodes (nodes with UUIDs) are *not*
          given new UUIDs.

        o Raise NoSuchWorkspaceException if 'srcWorspace' does not correspond
          to a known workspace within this repository.

        o Raise RepositoryException if the 'destAbsPath' provided has 
          an index on its final element.  Strictly speaking, the 'destAbsPath'
          parameter is actually an *absolute path* to the parent node of the
          new location, appended with the new *name* desired for the moved
          node. It does not specify a position within the child node ordering
          (if such ordering is supported).  If ordering is supported by the
          node type of the parent node of the new location, then the newly
          moved node is appended to the end of the child node list.

        o This method cannot be used to copy just an individual property by
          itself.  It copies an entire node and its subtree (including, of
          course, any properties contained therein).

        o Raise ConstraintViolationException if the operation would violate
          a node type or other implementation-specific constraint.

        o Raise VersionException if the parent node of 'destAbsPath' is
          versionable and checked-in, or is non-versionable
          but its nearest versionable ancestor is checked-in;  also if
          'removeExisting' is True, and a UUID conflict would require
          moving or altering a node that is checked in.

        o Raise AccessDeniedException if the current session (i.e. the session
          that was used to acquire this 'Workspace' object) does not have
          sufficient access rights to complete the operation.

        o Raise PathNotFoundException if the note at 'srcAbsPath' or the
          parent of 'destAbsPath' does not exist.

        o Raise ItemExistsException if a node or property already exists at
          'destAbsPath' or a node already exists there, and same-name siblings
          are not allowed, or if 'removeExisting' is False and a UUID
          conflict occurs..

        o Raise LockException if a lock prevents the clone.
        """

    def move(srcAbsPath, destAbsPath):
        """ Move the node at 'srcAbsPath' (and its entire subtree) to
            the new location at 'destAbsPath'.

        o 'srcAbsPath' is the path of the node to be moved.

        o 'destAbsPath' is the location to which the node at 'srcAbsPath'
          is to be moved.

        o If successful, the change is persisted immediately;  there is no
          need to call 'save'.

        o Raise RepositoryException if the provided 'destAbsPath' has an
          index on its final element.  Strictly speaking, the 'destAbsPath'
          parameter is actually an *absolute path* to the parent node of the
          new location, appended with the new *name* desired for the moved
          node. It does not specify a position within the child node ordering
          (if such ordering is supported).  If ordering is supported by the
          node type of the parent node of the new location, then the newly
          moved node is appended to the end of the child node list.

        o This method cannot be used to move just an individual property by
          itself.  It moves an entire node and its subtree (including, of
          course, any properties contained therein).

        o Raise ConstraintViolationException if performing this operation
          would violate a node type or implementation-specific constraint.

        o Raise VersionException if the parent node of 'destAbsPath' or the
          parent node of 'srcAbsPath' is versionable and checked-in, or is
          non-versionable and its nearest versionable ancestor is checked-in.

        o Raise AccessDeniedException if the current session (i.e. the session
          that was used to acquire this 'Workspace' object) does not have
          sufficient access rights to complete the operation.

        o Raise PathNotFoundException if no node exists at 'srcAbsPath' or
          no node exists one level above 'destAbsPath' (in other words,
          there is no node that will serve as the parent of the moved item).

        o Raise ItemExistsExceptionif a property already exists at
          'destAbsPath' or a node already exists there and same-name siblings
          are not allowed.

        o Raise LockException if a lock prevents the 'move'.
        """

    def getImportContentHandler(parentAbsPath, uuidBehavior):
        """ Return an 'xml.sax.handler.ContentHandler' which can be used to
            push SAX events into the repository.

        o 'parentAbsPath' is the absolute path of a node under which
          (as child) the imported subtree will be built.

        o 'uuidBehavior' is a flag that governs how incoming UUIDs
          are handled.  It must be one of the four values specified in the
          IImportUUIDBehavior enumeration.

        o If the incoming XML stream (in the form of SAX events) does not
          appear to be a JCR *system view* XML document then it is
          interpreted as a JCR *document view* XML document.

        o The incoming XML is deserialized into a subtree of items
          immediately below the node at 'parentAbsPath'.

        o This method simply returns the 'ContentHandler' without altering
          the state of the session;  the actual deserialization to the
          session transient space is done through the methods of the
          'ContentHandler'. Invalid XML data will cause the 'ContentHandler'
          to throw a 'SAXException'.

        o As SAX events are fed into the 'ContentHandler', changes are made
          directly at the workspace level, without going through the 'Session'.
          As a result, there is no need to call 'save'.  The advantage of
          this direct-to-workspace method is that a large import will not
          result in a large cache of pending nodes in the 'Session'.  The
          disadvantage is that structures that violate node type constraints
          cannot be imported, fixed and then saved.  Instead, a constraint
          violation will cause the 'ContentHandler' to throw a 'SAXException'.

          See 'Sesion.getImportContentHandler' for a version of this method
          that *does* go through the 'Session'.

        o The flag 'uuidBehavior' governs how the UUIDs of incoming
          (deserialized) nodes are handled. There are four options:

          'IImportUUIDBehavior.IMPORT_UUID_CREATE_NEW' --
            Incoming referenceable nodes are added in the same way that new
            node is added with 'Node.addNode'. That is, they are assigned
            newly created UUIDs upon addition to the workspace.  As a result
            UUID collisions never occur.

          'IImportUUIDBehavior.IMPORT_UUID_COLLISION_REMOVE_EXISTING' --
            If an incoming referenceable node has the same UUID as a node
            already existing in the workspace then the already existing node
            (and its subtree) is removed from wherever it may be in the
            workspace before the incoming node is added. Note that this can
            result in nodes *disappearing* from locations in the workspace
            that are remote from the location to which the incoming subtree
            is being written.

          'IImportUUIDBehavior.IMPORT_UUID_COLLISION_REPLACE_EXISTING' --
            If an incoming referenceable node has the same UUID as a node
            already existing in the workspace, then the already-existing node
            is replaced by the incoming node in the same position as the
            existing node. Note that this may result in the incoming subtree
            being disaggregated and 'spread around' to different locations
            in the workspace. In the most extreme case this behavior may
            result in no node at all being added as child of 'parentAbsPath'.
            This will occur if the topmost element of the incoming XML has the
            same UUID as an existing node elsewhere in the workspace.

          'IImportUUIDBehavior.IMPORT_UUID_COLLISION_THROW' --
            If an incoming referenceable node has the same UUID as a node
            already existing in the workspace then a 'SAXException' is thrown
            by the 'ContentHandler' during deserialization.

        o The returned 'ContentHandler' will raise an immediate 'SAXException'
          during deserialization if the top-most element of the incoming XML
          would deserialize to a node with the same name as an existing child
          of 'parentAbsPath' and that child does not allow same-name siblings.

        o The 'ContentHandler' raises 'SAXException' during deserialization
          if 'uuidBehavior' is set to 'IMPORT_UUID_COLLISION_REMOVE_EXISTING'
          and an incoming node has the same UUID as the node at
          'parentAbsPath' or one of its ancestors.

        o Raise PathNotFoundException if no node exists at 'parentAbsPath'.

        o Raise ConstraintViolationException if the new subtree cannot be
          added to the node at 'parentAbsPath' due to node-type or other
          implementation-specific constraints, and this can be determined
          before the first SAX event is sent.  Unlike
          'Session.getImportContentHandler' this method also enforces node
          type constraints by throwing 'SAXException's during deserialization.
          However, which node type constraints are enforced depends upon
          whether node type information in the imported data is respected,
          and this is an implementation-specific issue.  

        o Raise VersionException if the node at 'parentAbsPath' is versionable
          and checked-in, or is non-versionable but its nearest versionable
          ancestor is checked-in.

        o Raise LockException if a lock prevents the addition of the subtree.

        o Raise RepositoryException if another error occurs.

        o Raise AccessDeniedException if the current session (i.e. the session
          that was used to acquire this 'Workspace' object) does not have
          sufficient access rights to complete the operation.
        """

    def importXML(parentAbsPath, inputStream, uuidBehavior):
        """ Deserialize an XML document from 'inputStream' and add
            the resulting item subtree as a child of the node
            at 'parentAbsPath'.

        o 'parentAbsPath' is the absolute path of the node below which the
          deserialized subtree is added.

        o 'inputStream' an 'Inputstream' from which the XML to be
          deserilaized is read.

        o 'uuidBehavior' a four-value flag that governs how incoming UUIDs
           are handled.

        o If the incoming XML stream does not appear to be a JCR *system view*
          XML document then it is interpreted as a *document view*
          XML document.
          
        o Changes are made directly at the workspace level, without going
          through the 'Session'.  As a result, there is not need to call
          'save'.  The advantage of this direct-to-workspace method is that
          a large import will not result in a large cache of pending nodes
          in the 'Session'. The disadvantage is that invalid data cannot
          be imported, fixed and then saved. Instead, invalid data will cause
          this method to throw an * 'InvalidSerializedDataException'.
          
          See 'Session.importXML' for a version of this method that *does*
          go through the 'Session'.

        o The flag 'uuidBehavior' governs how the UUIDs of incoming
          (deserialized) nodes are handled. There are four options:

          'IImportUUIDBehavior.IMPORT_UUID_CREATE_NEW' --
            Incoming referenceable nodes are added in the same way that new
            node is added with 'Node.addNode'. That is, they are either
            assigned newly created UUIDs upon addition or upon 'save'
            (depending on the implementation). In either case, UUID collisions
            will not occur.

          'IImportUUIDBehavior.IMPORT_UUID_COLLISION_REMOVE_EXISTING' --
            If an incoming referenceable node has the same UUID as a node
            already existing in the workspace then the already existing node
            (and its subtree) is removed from wherever it may be in the
            workspace before the incoming node is added. Note that this can
            result in nodes *disappearing* from locations in the workspace
            that are remote from the location to which the incoming subtree
            is being written. Both the removal and the new addition
            will be persisted on 'save'.

          'IImportUUIDBehavior.IMPORT_UUID_COLLISION_REPLACE_EXISTING' --
            If an incoming referenceable node has the same UUID as a node
            already existing in the workspace, then the already-existing node
            is replaced by the incoming node in the same position as the
            existing node. Note that this may result in the incoming subtree
            being disaggregated and 'spread around' to different locations
            in the workspace. In the most extreme case this behavior may
            result in no node at all being added as child of 'parentAbsPath'.
            This will occur if the topmost element of the incoming XML has the
            same UUID as an existing node elsewhere in the workspace.
            The change will be persisted on 'save'.

          'IImportUUIDBehavior.IMPORT_UUID_COLLISION_THROW' --
            If an incoming referenceable node has the same UUID as a node
            already existing in the workspace then raise 'ItemExistsException'.

        o Raise 'ItemExistsException' if 'uuidBehavior' is set to
          'IMPORT_UUID_CREATE_NEW' or 'IMPORT_UUID_COLLISION_THROW' and the
          import would would overwrite an existing child of 'parentAbsPath'.

        o Raise IOException if an I/O error occurs on the stream.

        o Raise PathNotFoundException if no node exists at 'parentAbsPath'.

        o Raise ItemExisitsException if the top-most element of the incoming
          XML would deserialize to a node with the same name as an existing
          child of 'parentAbsPath' and that child does not allow same-name
          siblings, or if a 'uuidBehavior' is set to
          'IMPORT_UUID_COLLISION_THROW' and a UUID collision occurs.

        o Raise 'ConstraintViolationException if node-type or other
          implementation-specific constraints prevent the addition of the
          subtree.

        o Raise ConstraintViolationException if 'uuidBehavior' is set to
          'IMPORT_UUID_COLLISION_REMOVE_EXISTING' and an incoming node has the
          same UUID as the node at 'parentAbsPath' or one of its ancestors.

        o Raise VersionExceptionif the node at 'parentAbsPath' is versionable
          and checked-in, or is non-versionable but its nearest versionable
          ancestor is checked-in.

        o Raise LockExceptionif a lock prevents the addition of the subtree.

        o Raise RepositoryException if another error occurs.
        """

class IWorkspaceWithVersions(IWorkspace):
    """ API for optional version support
    """
    def restore(versions, removeExisting):
        """ Restore a set of versions at once.

        o 'versions' a sequence of 'IVersions' versions to be restored.

        o 'removeExisting' is a flag governing UUID collision policy.
        
        o Used in cases where a "chicken and egg" problem of mutually
          referring 'REFERENCE' properties would prevent the restore in any
          serial order.

        o If the restore succeeds the changes made to 'this' node are
          persisted immediately, there is no need to call 'save'.

        o The following restrictions apply to the set of versions specified:

          If 'S' is the set of versions being restored simultaneously,
     
          - For every version 'V' in 'S' that corresponds to
            a *missing* node, there must also be a parent of 'V' in 'S'.

          - 'S' must contain at least one version that corresponds to
            an existing node in the workspace.

          - No 'V' in 'S' can be a root version ('jcr:rootVersion').

        o Raise VersionException if any of these restrictions does not hold,
          because the system will be unable to determine the path locations
          to which one or more versions are to be restored.

        o The versionable nodes in this workspace that correspond to the
          versions being restored define a set of (one or more) subtrees.
          A UUID collision occurs when this workspace contains a node
          *outside these subtrees* that has the same UUID as one of the
          nodes that would be introduced by the 'restore' operation
          *into one of these subtrees*.

          The result in such a case is governed by the 'removeExisting' flag.
          If 'removeExisting' is True then the incoming node takes
          precedence, and the existing node (and its subtree) is removed.

          If a UUID dollision occurs, and 'removeExisting' is False,
          raise ItemExistsException (no changes are made).

          Note that this applies not only to cases where the restored
          node itself conflicts with an existing node but also to cases
          where a conflict occurs with any node that would be introduced into
          the workspace by the restore operation. In particular, conflicts
          involving subnodes of the restored node that have 'OnParentVersion'
          settings of * 'COPY' or 'VERSION' are also governed by the
          'removeExisting' flag.

        o Raise UnsupportedRepositoryOperationException if one or more of
          the nodes to be restored is not versionable.

        o Raise InvalidItemStateException if this 'Session' (not necessarily
          this 'Node') has pending unsaved changes.

        o Raise LockException' if a lock prevents the restore.
        """

class IWorkspaceWithLocking(IWorkspace):
    """ Extend IWorkspace to add optional locking support.
    """

    def addLockToken(lt):
        """ Add the specified lock token to this session.

        o 'lt' is a lock token (a string)
        
        o Holding a lock token allows the 'Session' object of the lock owner
          to alter nodes that are locked by the lock specified by that
          particular lock token.
        """

    def getLockTokens():
        """ Return a sequence of lock tokens currently held by this session.

        o Returned sequence elements are strings.
        """

    def removeLockTocken(lt):
        """ Remove the specified lock token from this session.

        o 'lt' is a lock token (a string)
        """

class IWorkspaceWithObservation(IWorkspace):
    """ Extend IWorkspace to add optional Observation support.
    """

    def getObservationManager():
        """ Return an 'IObservationManager' used to observe nodes in the
            workspace.

        o Raise UnsupportedRepositoryOperationException if the implementation
          does not support observation.

        o Raise RepositoryException if another error occurs.
        """
